'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _avsc = require('avsc');

var _avsc2 = _interopRequireDefault(_avsc);

var _envelope = require('./envelope.json');

var _envelope2 = _interopRequireDefault(_envelope);

var _envelopeHeaders = require('./envelope-headers.json');

var _envelopeHeaders2 = _interopRequireDefault(_envelopeHeaders);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

Function.prototype.$asyncbind = function anonymous(self, catcher) {
  var resolver = this;

  if (catcher === true) {
    if (!Function.prototype.$asyncbind.EagerThenable) Function.prototype.$asyncbind.EagerThenable = function factory(tick) {
      var _tasks = [];

      if (!tick) {
        try {
          tick = process.nextTick;
        } catch (ex) {
          tick = function (p) {
            setTimeout(p, 0);
          };
        }
      }

      function _untask() {
        for (var i = 0; i < _tasks.length; i += 2) {
          var t = _tasks[i + 1],
              r = _tasks[i];

          for (var j = 0; j < t.length; j++) t[j].call(null, r);
        }

        _tasks = [];
      }

      function isThenable(obj) {
        return obj && obj instanceof Object && typeof obj.then === 'function';
      }

      function EagerThenable(resolver) {
        function done(inline) {
          var w;
          if (_sync || phase < 0 || (w = _thens[phase]).length === 0) return;

          _tasks.push(result, w);

          _thens = [[], []];
          if (_tasks.length === 2) inline ? _untask() : tick(_untask);
        }

        function resolveThen(x) {
          if (phase >= 0) return;
          if (isThenable(x)) return x.then(resolveThen, rejectThen);
          phase = 0;
          result = x;
          done(true);
        }

        function rejectThen(x) {
          if (phase >= 0) return;
          if (isThenable(x)) return x.then(resolveThen, rejectThen);
          phase = 1;
          result = x;
          done(true);
        }

        function settler(resolver, rejecter) {
          _thens[0].push(resolver);

          _thens[1].push(rejecter);

          done();
        }

        function toString() {
          return 'EagerThenable{' + {
            '-1': 'pending',
            0: 'resolved',
            1: 'rejected'
          }[phase] + '}=' + result.toString();
        }

        function guard() {
          try {
            resolver.call(null, resolveThen, rejectThen);
          } catch (ex) {
            rejectThen(ex);
          }
        }

        this.then = settler;
        this.toString = toString;
        var _thens = [[], []],
            _sync = true,
            phase = -1,
            result;
        guard();
        _sync = false;
        done();
      }

      EagerThenable.resolve = function (v) {
        return isThenable(v) ? v : {
          then: function (resolve, reject) {
            return resolve(v);
          }
        };
      };

      return EagerThenable;
    }();
    return new Function.prototype.$asyncbind.EagerThenable(boundThen);
  }

  if (catcher) {
    if (Function.prototype.$asyncbind.wrapAsyncStack) catcher = Function.prototype.$asyncbind.wrapAsyncStack(catcher);
    return then;
  }

  function then(result, error) {
    try {
      return result && result instanceof Object && typeof result.then === 'function' ? result.then(then, catcher) : resolver.call(self, result, error || catcher);
    } catch (ex) {
      return (error || catcher)(ex);
    }
  }

  function boundThen(result, error) {
    return resolver.call(self, result, error);
  }

  boundThen.then = boundThen;
  return boundThen;
};

/**
 * The AvroFormat is used when working with a stream that contains a serialized
 * Apache Avro schema. The AvroFormat uses the AVSC library for all it's work with
 * the formatting.
 */
class AvroFormat {

  /**
   * Initialize a new instance of the AvroFormat.
   */
  constructor(registry) {
    this.envelopeSchema = _avsc2.default.parse(_envelope2.default);
    this.headerSchema = _avsc2.default.parse(_envelopeHeaders2.default);
    this.headerResolver = this.headerSchema.createResolver(this.envelopeSchema);
    this.registry = registry;
  }

  /**
   * Extract the message type from the headers in the stream.
   */
  decodeMessageType(rawMessage) {
    const message = this.headerSchema.fromBuffer(rawMessage, this.headerResolver, true);
    return message.headers.schemaFingerprint.toString('hex');
  }

  /**
   * Decode a message
   */
  decodeMessage(rawMessage) {
    return new _bluebird2.default(function ($return, $error) {
      var envelopeContent, fingerprint, innerSchema, decoded;
      envelopeContent = this.envelopeSchema.fromBuffer(rawMessage);
      fingerprint = envelopeContent.headers.schemaFingerprint.toString('hex');
      return this.registry.getSchema(fingerprint).then(function ($await_1) {
        innerSchema = $await_1;
        decoded = innerSchema.fromBuffer(envelopeContent.binaryData, null, true);


        // Strip away the AVSC magic. Immense hack.
        Object.setPrototypeOf(decoded, Object.prototype);

        return $return(decoded);
      }.$asyncbind(this, $error), $error);
    }.$asyncbind(this));
  }

  /**
   * Encode the given message type for storage in a stream.
   */
  encodeMessage(typeKey, message) {
    return new _bluebird2.default(function ($return, $error) {
      var innerSchema, innerBuffer, wrapper, buffer;
      return this.registry.getSchema(typeKey).then(function ($await_2) {
        innerSchema = $await_2;
        innerBuffer = innerSchema.toBuffer(message);
        wrapper = {
          headers: {
            schemaFingerprint: new Buffer(typeKey, 'hex')
          },
          binaryData: innerBuffer
        };
        buffer = this.envelopeSchema.toBuffer(wrapper);


        return $return(buffer);
      }.$asyncbind(this, $error), $error);
    }.$asyncbind(this));
  }
}
exports.default = AvroFormat;
//# sourceMappingURL=index.js.map
